﻿Type=Class
Version=2.02
@EndOfDesignText@
'Class module
Private Sub Class_Globals
	Type MyTextbox(Name As String, DataType As Int, ActionBtn As Int, ActionSub As String, ActionSubModule As Object, MinChar As Int, MaxChar As Int)
	Type MyActionSub(ActionSub As String, ActionSubModule As Object)
	Private IME As IME ' Put here instead of below so class is considered an Activity
	Private MyViews As List
	Private NameMap As Map
	Private CurFocus As View
	Public Padding As Int
	Public MinMaxWarn As Boolean
	Public ScaleViews As Boolean
	Public OverrideFontScale As Boolean
	Private FontScale As Float
	Private ScaleLabel As Label
	Private Parent As Panel
	Public ParentWidth As Int
	Private CurX As Int
	Private NextX As Int
	Private CurY As Int
	Private NextY As Int
End Sub

' Initialize the Class
' Container: Panel to place all controls in
' ShowMinMaxWarn: Set if Toast Messages are to show when Min and Max Char requirements aren't met
' ViewScaling: Sets if Views are to be scaled to fit the text they contain
' OverrideFontScaling: Set if You wish to override the user's font size scale
Public Sub Initialize(Container As Panel, ContainerWidth As Int, ShowMinMaxWarn As Boolean, ViewScaling As Boolean, OverrideFontScaling As Boolean)
Dim Ref As Reflector

	MyViews.Initialize
	NameMap.Initialize
	IME.Initialize("IME")
	ScaleLabel.Initialize("")
	ScaleLabel.Visible = False
	Container.AddView(ScaleLabel, 0, 0, 1, 1)
	Padding = 5dip
	MinMaxWarn = ShowMinMaxWarn
	ScaleViews = ViewScaling
	OverrideFontScale = OverrideFontScaling
	Parent = Container
	ParentWidth = ContainerWidth
	CurX = 5dip
	NextX = 5dip
	CurY = 5dip
	NextY = 5dip
	
	Ref.Target = Ref.GetContext
	Ref.Target = Ref.RunMethod("getResources")
	Ref.Target=Ref.RunMethod("getConfiguration")
	FontScale= Ref.GetField("fontScale")
End Sub

#Region Constants
' General Text Data [Type_Class_Text]
Sub DataType_Text As Int
	Return 1
End Sub

' Numbers Only [Type_Class_Number]
Sub DataType_Num As Int
	Return 2
End Sub

' Floating Point Numbers with sign and decimal [Type_Class_Number | Type_Number_Flag_Decimal | Type_Number_Flag_Signed]
Sub DataType_Float As Int
	Return 12290
End Sub

' DateTime Text [Type_Class_DateTime]
Sub DataType_DateTime As Int
	Return 4
End Sub

' Date Text [Type_Class_DateTime | Type_DateTime_Variation_Date]
Sub DataType_Date As Int
	Return 20
End Sub

' Time Text [Type_Class_DateTime | Type_DateTime_Variation_Time]
Sub DataType_Time As Int
	Return 36
End Sub

' Password [Type_Class_Text | Type_Text_Variation_Password]
Sub DataType_Password As Int
	Return 129
End Sub

' Email Text [Type_Class_Text | Type_Text_Variation_Email_Address]
Sub DataType_Email As Int
	Return 33
End Sub

' URL Text [Type_Class_Text | Type_Text_Variation_URI]
Sub DataType_URL As Int
	Return 17
End Sub

' Phone Number Text [Type_Class_Phone]
Sub DataType_Phone As Int
	Return 3
End Sub

' Name Text [Type_Class_Text | Type_Text_Variation_Person_Name | Type_Text_Flag_Cap_Words]
Sub DataType_Name As Int
	Return 8289
End Sub

' Uppercase Every Char [Type_Class_Text | Type_Text_Cap_Characters]
' (Overrides all other Uppercase Flags)
Sub DataType_Uppercase_All As Int
	Return 4097
End Sub

' Uppercase First Letter of Each Sentence [Type_Class_Text | Type_Text_Cap_Sentences]
Sub DataType_Uppercase_Sentences As Int
	Return 16385
End Sub

' Uppercase First Letter of Each Word [Type_Class_Text | Type_Text_Cap_Words]
' (Overrides Sentences Uppercase Flag)
Sub DataType_Uppercase_Words As Int
	Return 8193
End Sub

' Text should have Auto Correction Applied [Type_Class_Text | Type_Text_Flag_Auto_Correct]
Sub DataType_Auto_Correct As Int
	Return 32769
End Sub

' No Suggestions/Auto Correct [Type_Class_Text | Type_Text_Flag_No_Suggestions]
Sub DataType_No_Suggestions As Int
	Return 524289
End Sub

' Shows Next Button and will navigate to Next View
Sub ActionBtn_Next As Int
	Return 5
End Sub

' Shows Previous Button and will navigate to Previous View
Sub ActionBtn_Previous As Int
	Return 7
End Sub

' Shows Done Button and has option to execute ActionSub
Sub ActionBtn_Done As Int
	Return 6
End Sub

' Shows Go Button and has option to execute ActionSub
Sub ActionBtn_Go As Int
	Return 2
End Sub

' Shows Search Button and has option to execute ActionSub
Sub ActionBtn_Search As Int
	Return 3
End Sub

' Shows Send Button and has option to execute ActionSub
Sub ActionBtn_Send As Int
	Return 4
End Sub

' Upper and Lower Case Alpha Chars
' Returns: "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
Sub CharFilter_Alpha As String
	Return "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
End Sub

' Uppercase Alpha Chars
' Returns: "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
Sub CharFilter_Upper_Alpha As String
	Return "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
End Sub

' Number Chars
' Returns: "0123456789"
Sub CharFilter_Numeric As String
	Return "0123456789"
End Sub
#End Region

' Add a view of your own (Not Managed by the Class) to the Flow/Layout of views managed by the class
' Left/Top: Position to place view, -1 to Place in Current Flow Position, -2 to Place in Next Flow Position.
' Width/Height: Width/Height of View. -1 to Fit to Full Width/Height. -2 to -99 for percent of width
Sub Add2Flow(NewView As View, Left As Int, Top As Int, Width As Int, Height As Int) As Rect
Dim viewRect As Rect

	If Top = -2 Then
		Top = NextY
	Else If Top = -1 Then
		Top = CurY
		If Left > 0 AND Left < NextX Then Left = NextX
	End If
	If Left = -2 Then
		Left = NextX
	Else If Left = -1 Then
		Left = CurX
	End If
	
	If Width < 1 Then
		If Width < -1 AND Width > -100 Then Width = Abs(Width) * .01 * (ParentWidth - Padding - Left) Else Width = ParentWidth - Padding - Left
	End If
	If Height < 1 Then
		If Height < -1 AND Height > -100 Then Height = Abs(Height) * .01 * (Parent.Height - Padding - Top) Else Height = Parent.Height - Padding - Top
	End If
	
	Parent.AddView(NewView, Left, Top, Width, Height)
	
	CurX = Left
	NextX = Left + NewView.Width + Padding
	CurY = Top
	NextY = Max(NextY, Top + NewView.Height + Padding)
	If NextY > Parent.Height Then Parent.Height = NextY
	
	viewRect.Initialize(Left, Top, Left + NewView.Width, Top + NewView.Height)
	Return viewRect
End Sub

' Left/Top: Position to place view, -1 to Place in Current Flow Position, -2 to Place in Next Flow Position.
' Width/Height: Width/Height of View. If ScaleViews is True it will adjust to fit font Height (Set Width= 1 to Scale width). -1 to Fit to Full Width/Height. -2 to -99 for percent of width.
' Text: Text of Label.
' Color: Color of Label
' LinkedView: Name String for View that will get focus when Label is Clicked
Sub AddLabel(Left As Int, Top As Int, Width As Int, Height As Int, Text As String, TextSize As Float, Color As Int, LinkedView As String) As Rect
Dim myControl As Label
Dim viewRect As Rect
Dim ref As Reflector

	myControl.Initialize("Label")
	myControl.Gravity = Bit.Or(Gravity.Left, Gravity.Top)
	myControl.Text = Text
	myControl.TextColor = Color
	myControl.TextSize = TextSize
	myControl.Typeface = Typeface.DEFAULT_BOLD
	ref.Target = myControl
	ref.RunMethod2("setLines", 1, "java.lang.int")
	ref.RunMethod2("setHorizontallyScrolling", True, "java.lang.boolean")
	ref.RunMethod2("setEllipsize", "MARQUEE", "android.text.TextUtils$TruncateAt")
	ref.RunMethod2("setMarqueeRepeatLimit", -1, "java.lang.int")
	ref.RunMethod2("setSelected", True, "java.lang.boolean")
	
	myControl.Tag = LinkedView
	
	If Top = -2 Then
		Top = NextY
	Else If Top = -1 Then
		Top = CurY
		If Left > 0 AND Left < NextX Then Left = NextX
	End If
	If Left = -2 Then
		Left = NextX
	Else If Left = -1 Then
		Left = CurX
	End If
	
	If Width < 1 Then
		If Width < -1 AND Width > -100 Then Width = Abs(Width) * .01 * (ParentWidth - Padding - Left) Else Width = ParentWidth - Padding - Left
	End If
	If Height < 1 Then
		If Height < -1 AND Height > -100 Then Height = Abs(Height) * .01 * (Parent.Height - Padding - Top) Else Height = Parent.Height - Padding - Top
	End If
	
	If OverrideFontScale = True Then myControl.TextSize = myControl.TextSize / FontScale
	
	If ScaleViews = True Then ' Scale View to Size required for Font
		Dim charRect As Rect
		charRect = GetCharSize(myControl.Typeface, myControl.TextSize, Text)
		
		If Height < charRect.Bottom Then Height = charRect.Bottom
		If Text.Length > 0 AND Width = 1 Then ' Calculate Min Width
			Width = Min(charRect.Right, ParentWidth - Padding - Left)
		Else
			Width = Min(Width, ParentWidth - Padding - Left)
		End If
	End If
	
	Parent.AddView(myControl, Left, Top, Width, Height)
	
	CurX = Left
	NextX = Left + myControl.Width + Padding
	CurY = Top
	NextY = Max(NextY, Top + myControl.Height + Padding)
	If NextY > Parent.Height Then Parent.Height = NextY
	
	viewRect.Initialize(Left, Top, Left + myControl.Width, Top + myControl.Height)
	Return viewRect
End Sub

Private Sub Label_Click
Dim Name As String
Dim MyControl As Label
Dim LV As View
	
	MyControl = Sender
	Name = MyControl.Tag
	If Name.Length > 0 Then
		LV = NameMap.Get(Name)
		If LV.IsInitialized = True Then
			If CurFocus = LV Then
				If LV Is AutoCompleteEditText Then
					Dim act As AutoCompleteEditText
					act = LV
					act.ShowDropDown
				End If
			Else
				LV.RequestFocus
				CurFocus = LV
			End If
		End If
	End If
End Sub

' Left/Top: Position to place view, -1 to Place in Current Flow Position, -2 to Place in Next Flow Position.
' Width/Height: Width/Height of View. If ScaleViews is True it will adjust to fit font Height (Set Width= 1 to Scale width). -1 to Fit to Full Width/Height. -2 to -99 for percent of width.
' Text: Text of Label.
' Color: Color of Label
' ActionSub: Name of Sub to call when Button Clicked
' ActionSubModule: Activity/Module that Sub is in.  (Usually [Me] can be used if Sub is in Activity Module the View resides in).
Sub AddButton(Left As Int, Top As Int, Width As Int, Height As Int, Name As String, Text As String, TextSize As Float, Color As Int, ActionSub As String, ActionSubModule As Object) As Rect
Dim myControl As Label
Dim viewRect As Rect
Dim ref As Reflector
Dim newActionSub As MyActionSub

	myControl.Initialize("Button")
	myControl.Gravity = Gravity.CENTER
	myControl.Text = Text
	myControl.TextColor = Color
	myControl.TextSize = TextSize
	myControl.Typeface = Typeface.DEFAULT_BOLD
	newActionSub.Initialize
	newActionSub.ActionSub = ActionSub
	newActionSub.ActionSubModule = ActionSubModule
	myControl.Tag = newActionSub
	
	If Top = -2 Then
		Top = NextY
	Else If Top = -1 Then
		Top = CurY
		If Left > 0 AND Left < NextX Then Left = NextX
	End If
	If Left = -2 Then
		Left = NextX
	Else If Left = -1 Then
		Left = CurX
	End If
	
	If Width < 1 Then
		If Width < -1 AND Width > -100 Then Width = Abs(Width) * .01 * (ParentWidth - Padding - Left) Else Width = ParentWidth - Padding - Left
	End If
	If Height < 1 Then
		If Height < -1 AND Height > -100 Then Height = Abs(Height) * .01 * (Parent.Height - Padding - Top) Else Height = Parent.Height - Padding - Top
	End If
	
	If OverrideFontScale = True Then myControl.TextSize = myControl.TextSize / FontScale
	
	If ScaleViews = True Then ' Scale View to Size required for Font
		Dim charRect As Rect
		charRect = GetCharSize(myControl.Typeface, myControl.TextSize, Text)
		
		If Height < charRect.Bottom + 15dip Then Height = charRect.Bottom + 15dip
		If Text.Length > 0 AND Width = 1 Then ' Calculate Min Width
			Width = Min(charRect.Right + 15dip, ParentWidth - Padding - Left)
		Else
			Width = Min(Width, ParentWidth - Padding - Left)
		End If
	End If
	
	Parent.AddView(myControl, Left, Top, Width, Height)
	myControl.Background = SetNinePatchDrawable("buttonup")
	ref.Target = myControl
	ref.SetOnTouchListener("Button_Touch")
	
	CurX = Left
	NextX = Left + myControl.Width + Padding
	CurY = Top
	NextY = Max(NextY, Top + myControl.Height + Padding)
	If NextY > Parent.Height Then Parent.Height = NextY
	If Name.Length > 0 Then NameMap.Put(Name, myControl)
	
	viewRect.Initialize(Left, Top, Left + myControl.Width, Top + myControl.Height)
	Return viewRect
End Sub

Private Sub Button_Click
Dim myControl As Label
Dim tagActionSub As MyActionSub

	myControl = Sender
	tagActionSub = myControl.Tag
	If tagActionSub.IsInitialized Then
		If tagActionSub.ActionSub.Length > 0 Then CallSubDelayed(tagActionSub.ActionSubModule, tagActionSub.ActionSub)
	End If
End Sub

Private Sub Button_Touch(viewtag As Object, action As Int, x As Float, y As Float, moveEvent As Object) As Boolean
Dim myControl As Label

	myControl = Sender
	If action = 0 Then myControl.Background = SetNinePatchDrawable("buttondown")
	If action = 1 OR x < 0 OR y < 0 OR x > myControl.Width OR y > myControl.Height Then myControl.Background = SetNinePatchDrawable("buttonup")
End Sub

' Left/Top: Position to place view, -1 to Place in Current Flow Position, -2 to Place in Next Flow Position.
' Width/Height: Width/Height of View. If ScaleViews is True it will adjust to fit font Height (Set Width= 1 to Scale width to ViewChar). -1 to Fit to Full Width/Height. -2 to -99 for percent of width.
' Name: Name given to the control for reference or use in a Select Case Block.
' ActionSub: Name of Sub to call when Item Selected- Sub YourSub(Position As Int, Value As Object). {Matches the ItemClick for Spinner}
' ActionSubModule: Activity/Module that Sub is in.  (Usually [Me] can be used if Sub is in Activity Module the View resides in).
' ViewChar: Set to desired number of chars visible.
Public Sub AddComboBox(Left As Int, Top As Int, Width As Int, Height As Int, Name As String, Prompt As String, Items As List, TextSize As Float, ActionSub As String, ActionSubModule As Object, ViewChar As Int) As Rect
Dim myControl As Spinner
Dim ref As Reflector
Dim viewRect As Rect
Dim newActionSub As MyActionSub

	newActionSub.Initialize
	myControl.Initialize("Combobox")
	newActionSub.ActionSub = ActionSub
	newActionSub.ActionSubModule = ActionSubModule
	myControl.Tag = newActionSub
	myControl.Prompt = Prompt
	myControl.TextSize = TextSize
	
	If Top = -2 Then
		Top = NextY
	Else If Top = -1 Then
		Top = CurY
		If Left > 0 AND Left < NextX Then Left = NextX
	End If
	If Left = -2 Then
		Left = NextX
	Else If Left = -1 Then
		Left = CurX
	End If
	
	If Width < 1 Then
		If Width < -1 AND Width > -100 Then Width = Abs(Width) * .01 * (ParentWidth - Padding - Left) Else Width = ParentWidth - Padding - Left
	End If
	If Height < 1 Then
		If Height < -1 AND Height > -100 Then Height = Abs(Height) * .01 * (Parent.Height - Padding - Top) Else Height = Parent.Height - Padding - Top
	End If
	
	myControl.TextSize = myControl.TextSize / FontScale ' Had to override font scale no matter what for spinner/combobox otherwise it draws odd
	
	If ScaleViews = True Then ' Scale View to Size required for Font
		Dim charRect As Rect
		charRect = GetCharSize(Typeface.DEFAULT, myControl.TextSize, "")
		If Height < charRect.Bottom + 11dip + myControl.TextSize Then Height = charRect.Bottom + 11dip + myControl.TextSize
		If ViewChar > 0 AND Width < 2 Then ' Calculate Min Width
			Width = Min(charRect.Right * ViewChar + 50dip, ParentWidth - Padding - Left)
		Else
			Width = Min(Width, ParentWidth - Padding - Left)
		End If
	End If
	
	Parent.AddView(myControl, Left, Top, Width, Height)
	myControl.Background = SetNinePatchDrawable("comboboxenabled")
	ref.Target = myControl
	ref.RunMethod4("setPadding", Array As Object(6dip, 6dip, 44dip, 5dip), Array As String("java.lang.int", "java.lang.int", "java.lang.int", "java.lang.int"))
	
	CurX = Left
	NextX = Left + myControl.Width + Padding
	CurY = Top
	NextY = Max(NextY, Top + myControl.Height + Padding)
	If NextY > Parent.Height Then Parent.Height = NextY
	
	If Items.IsInitialized Then myControl.AddAll(Items)
	MyViews.Add(myControl)
	If Name.Length > 0 Then NameMap.Put(Name, myControl)
	
	viewRect.Initialize(Left, Top, Left + myControl.Width, Top + myControl.Height)
	Return viewRect
End Sub

Private Sub Combobox_ItemClick (Position As Int, Value As Object)
Dim myControl As Spinner
Dim tagActionSub As MyActionSub

	myControl = Sender
	tagActionSub = myControl.Tag
	If tagActionSub.IsInitialized Then
		If tagActionSub.ActionSub.Length > 0 Then CallSubDelayed3(tagActionSub.ActionSubModule, tagActionSub.ActionSub, Position, Value)
	End If
End Sub

' Left/Top: Position to place view, -1 to Place in Current Flow Position, -2 to Place in Next Flow Position.
' Width/Height: Width/Height of View. If ScaleViews is True it will adjust to fit font (1 for Min and 2 for Max Char Width). -1 to Fit to Full Width/Height. -2 to -99 for percent of width.
' Name: Name given to the control for reference or use in a Select Case Block.
' DataType: Use one of the Constants within this Class to Specify the Data Type for the View (Can combine with OR).
' CharFilter: Use one of the Constants within this Class and/or your own string to Specify the Text Filter for the View's Allowed Chars (Join Strings with &amp;).
' ActionBtn: Use one of the Constants within this Class to Specify the Action Button for the View's Keyboard.
' ActionSub: Name of Sub to call for Action Buttons other than Next and Previous.
' ActionSubModule: Activity/Module that Sub is in.  (Usually [Me] can be used if Sub is in Activity Module the View resides in).
' Min/Max Char: Set to desired value or 0 for no restriction.
Public Sub AddTextBox(Left As Int, Top As Int, Width As Int, Height As Int, Name As String, Text As String, TextSize As Float, Hint As String, MultiLine As Boolean, DataType As Int, CharFilter As String, ActionBtn As Int, ActionSub As String, ActionSubModule As Object, MinChar As Int, MaxChar As Int, AutoCompleteItems As List) As Rect
Dim myControl As AutoCompleteEditText
Dim ref As Reflector
Dim viewRect As Rect
Dim newTextBox As MyTextbox

	newTextBox.Initialize
	myControl.Initialize("Edit")
	newTextBox.DataType = DataType
	newTextBox.ActionBtn = ActionBtn
	newTextBox.ActionSub = ActionSub
	newTextBox.ActionSubModule = ActionSubModule
	newTextBox.MinChar = MinChar
	newTextBox.MaxChar = MaxChar
	newTextBox.Name = Name
	myControl.Tag = newTextBox
	myControl.Text = Text
	myControl.Hint = Hint
	myControl.Typeface = Typeface.DEFAULT
	myControl.TextSize = TextSize
	
	If Top = -2 Then
		Top = NextY
	Else If Top = -1 Then
		Top = CurY
		If Left > 0 AND Left < NextX Then Left = NextX
	End If
	If Left = -2 Then
		Left = NextX
	Else If Left = -1 Then
		Left = CurX
	End If
	
	If Width < 1 Then
		If Width < -1 AND Width > -100 Then Width = Abs(Width) * .01 * (ParentWidth - Padding - Left) Else Width = ParentWidth - Padding - Left
	End If
	If Height < 1 Then
		If Height < -1 AND Height > -100 Then Height = Abs(Height) * .01 * (Parent.Height - Padding - Top) Else Height = Parent.Height - Padding - Top
	End If
	
	ref.Target = myControl
	If MultiLine Then
		myControl.SingleLine = False
		myControl.Wrap= True
		newTextBox.DataType = Bit.Or(newTextBox.DataType, 131072) ' Multiline Text Flag
		ref.RunMethod2("setImeOptions", Bit.Or(1073741824, newTextBox.ActionBtn), "java.lang.int") 'flagNoEnterAction= 1073741824
	Else
		myControl.SingleLine = True
		myControl.Wrap= False
		ref.RunMethod2("setImeOptions", Bit.Or(268435456, newTextBox.ActionBtn), "java.lang.int") 'flagNoExtractUi= 268435456
	End If
	myControl.InputType = newTextBox.DataType
	
	If OverrideFontScale = True Then myControl.TextSize = myControl.TextSize / FontScale
	
	If ScaleViews = True Then ' Scale View to Size required for Font
		Dim charRect As Rect
		charRect = GetCharSize(myControl.Typeface, myControl.TextSize, "")
		If Height < charRect.Bottom + 7dip Then Height = charRect.Bottom + 7dip
		If Width < 3 Then ' Calculate Min Width
			If Width = 1 Then
				Width = Min(charRect.Right * MinChar + 7dip, ParentWidth - Padding - Left)
			Else ' Width = 2
				Width = Min(charRect.Right * MaxChar + 7dip, ParentWidth - Padding - Left)
			End If
		Else
			Width = Min(Width, ParentWidth - Padding - Left)
		End If
	End If
	
	Parent.AddView(myControl, Left, Top, Width, Height)
	myControl.Background = SetNinePatchDrawable("editenabled")
	
	CurX = Left
	NextX = Left + myControl.Width + Padding
	CurY = Top
	NextY = Max(NextY, Top + myControl.Height + Padding)
	If NextY > Parent.Height Then Parent.Height = NextY
	
	If CharFilter.Length = 0 Then
		If Bit.And(DataType, DataType_Phone) = DataType_Phone Then
			CharFilter = CharFilter_Numeric & "*#+-P()N,/ "
		Else If Bit.And(DataType, DataType_URL) = DataType_URL Then
			CharFilter = CharFilter_Alpha & CharFilter_Numeric & "$.,'-+!*_~:%()"
		Else If Bit.And(DataType, DataType_Email) = DataType_Email Then
			CharFilter = CharFilter_Alpha & CharFilter_Numeric & "!#$%&'*+-/=?^_`{|}~."
		Else If Bit.And(DataType, DataType_Float) = DataType_Float Then
			CharFilter = CharFilter_Numeric & ".-"
		Else If Bit.And(DataType, DataType_Num) = DataType_Num Then
			CharFilter = CharFilter_Numeric
		End If
	End If
	If CharFilter.Length > 0 Then IME.SetCustomFilter(myControl, myControl.InputType, CharFilter)
	
	IME.AddHandleActionEvent(myControl)
	
	If AutoCompleteItems.IsInitialized Then myControl.SetItems(AutoCompleteItems)
	MyViews.Add(myControl)
	If Name.Length > 0 Then NameMap.Put(Name, myControl)
	
	viewRect.Initialize(Left, Top, Left + myControl.Width, Top + myControl.Height)
	Return viewRect
End Sub

' Get Text of Named Views
Public Sub GetText(Name As String) As String
Dim FoundName As View

	FoundName = NameMap.Get(Name)
	If FoundName.IsInitialized = False Then
		Return ""
	Else
		If FoundName Is Spinner Then
			Dim myCombobox As Spinner
			myCombobox = FoundName
			Return myCombobox.SelectedItem
		Else If FoundName Is Label Then
			Dim myEdit As Label
			myEdit = FoundName
			Return myEdit.Text
		Else
			Return ""
		End If
	End If
End Sub

' Set Text of Named Views
Public Sub SetText(Name As String, Value As String)
Dim FoundName As View

	FoundName = NameMap.Get(Name)
	If FoundName.IsInitialized = True Then
		If FoundName Is Spinner Then
			Dim myCombobox As Spinner
			Dim selection As Int
			myCombobox = FoundName
			selection = myCombobox.IndexOf(Value)
			If selection  > -1 Then myCombobox.SelectedIndex = selection
		Else If FoundName Is Label Then
			Dim myLabel As Label
			myLabel = FoundName
			myLabel.Text = Value
		End If
	End If
End Sub

' Get Map of all Named Views Texts
Public Sub GetAllText As Map
Dim newMap As Map
Dim curValue As String

	newMap.Initialize
	For i = 0 To NameMap.Size - 1
		If NameMap.GetValueAt(i) Is Spinner Then
			Dim myCombobox As Spinner
			myCombobox = NameMap.GetValueAt(i)
			curValue = myCombobox.SelectedItem
		Else If NameMap.GetValueAt(i) Is Label Then
			Dim myEdit As Label
			myEdit = NameMap.GetValueAt(i)
			curValue = myEdit.Text
		Else
			curValue = ""
		End If
		newMap.Put(NameMap.GetKeyAt(i), curValue)
	Next
	Return newMap
End Sub

' Set Text of all Named Views with Provided Map
Public Sub SetAllText(Texts As Map)
Dim myView As View

	For i = 0 To Texts.Size - 1
		myView = NameMap.Get(Texts.GetKeyAt(i))
		If myView.IsInitialized Then
			If myView Is Spinner Then
				Dim myCombobox As Spinner
				Dim selection As Int
				myCombobox = myView
				selection = myCombobox.IndexOf(Texts.GetValueAt(i))
				If selection  > -1 Then myCombobox.SelectedIndex = selection
			Else If myView Is Label Then
				Dim myEdit As Label
				myEdit = myView
				myEdit.Text = Texts.GetValueAt(i)
			End If
		End If
	Next
End Sub

' Set Text of all Named Views with Provided Cursor
Public Sub SetAllTextDB(dbCursor As Cursor)
Dim myView As View
	If dbCursor.RowCount > 0 Then
		dbCursor.Position = 0
		For i = 0 To dbCursor.ColumnCount - 1
			myView = NameMap.Get(dbCursor.GetColumnName(i))
			If myView.IsInitialized Then
				If myView Is Spinner Then
					Dim myCombobox As Spinner
					Dim selection As Int
					myCombobox = myView
					selection = myCombobox.IndexOf(dbCursor.GetString2(i))
					If selection  > -1 Then myCombobox.SelectedIndex = selection
				Else If myView Is Label Then
					Dim myEdit As Label
					myEdit = myView
					myEdit.Text = dbCursor.GetString2(i)
				End If
			End If
		Next
	End If
End Sub

' Get Copy of the Map of all Named Views
Public Sub GetViews As Map
Dim newMap As Map

	newMap.Initialize
	For i = 0 To NameMap.Size - 1
		newMap.Put(NameMap.GetKeyAt(i), NameMap.GetValueAt(i))
	Next
	Return newMap
End Sub

' Get the View with the given Name
Public Sub GetView(Name As String) As View
	Return NameMap.Get(Name)
End Sub

' Set Enabled status of Named View
Public Sub Enabled(Name As String, EnableView As Boolean)
Dim FoundName As View

	FoundName = NameMap.Get(Name)
	If FoundName.IsInitialized = True Then
		If FoundName Is Spinner Then
			FoundName.Enabled = EnableView
			If EnableView = True Then
				FoundName.Background = SetNinePatchDrawable("comboboxenabled")
			Else
				FoundName.Background = SetNinePatchDrawable("comboboxdisabled")
			End If
		Else If FoundName Is EditText Then
			Dim ref As Reflector

			FoundName.Enabled = EnableView
			ref.Target = FoundName
			If EnableView = True Then
				FoundName.Background = SetNinePatchDrawable("editenabled")
				ref.RunMethod2("setFocusable", "True", "java.lang.boolean")
				ref.RunMethod2("setFocusableInTouchMode", "True", "java.lang.boolean")
			Else
				FoundName.Background = SetNinePatchDrawable("editdisabled")
				ref.RunMethod2("setFocusable", "False", "java.lang.boolean")
				ref.RunMethod2("setFocusableInTouchMode", "False", "java.lang.boolean")
			End If
		Else If FoundName Is Label Then
			FoundName.Enabled = EnableView
		End If
	End If
End Sub

' Get the Max Width and Height of a Char or of SampleText if given
Public Sub GetCharSize(ViewTypeface As Typeface, ViewFontsize As Float, SampleText As String) As Rect
Dim testCanvas As Canvas
Dim strUtil As StringUtils
Dim testString As String
Dim CharFrame As Rect

	ScaleLabel.Typeface = ViewTypeface
	ScaleLabel.TextSize = ViewFontsize
	testCanvas.Initialize(ScaleLabel)
	CharFrame.Initialize(0,0,0,0)
	If SampleText.Length = 0 Then testString = CharFilter_Alpha & CharFilter_Numeric & "`~!@#$%^&*()_+=-[]\\{}|;':,./<>?" & QUOTE Else testString = SampleText
	ScaleLabel.Width = testCanvas.MeasureStringWidth(testString, ScaleLabel.Typeface, ScaleLabel.TextSize) + 2dip
	CharFrame.Bottom = strUtil.MeasureMultilineTextHeight(ScaleLabel, testString)
	If SampleText.Length > 0 Then
		CharFrame.Right = ScaleLabel.Width
	Else
		For x = 0 To testString.Length - 1
			CharFrame.Right = Max(CharFrame.Right, testCanvas.MeasureStringWidth(testString.CharAt(x), ScaleLabel.Typeface, ScaleLabel.TextSize))
		Next
	End If
	Return CharFrame
End Sub

Private Sub SetNinePatchDrawable(ImageName As String) As Object
    Dim r As Reflector
    Dim package As String
    Dim id As Int
    package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    id = r.GetStaticField(package & ".R$drawable", ImageName)
    r.Target = r.GetContext
    r.Target = r.RunMethod("getResources")
    Return r.RunMethod2("getDrawable", id, "java.lang.int")
End Sub

' Test if Value in View meets Min Character Requirements
Public Sub isValid(Name As String) As Boolean
Dim FoundName As View

	FoundName = NameMap.Get(Name)
	If FoundName.IsInitialized = True Then
		If FoundName Is Spinner Then
			Dim myCombobox As Spinner
			myCombobox = FoundName
			If myCombobox.SelectedIndex > -1 Then Return True Else Return False
		Else If FoundName Is EditText Then
			Dim myEdit As EditText
			Dim MTB As MyTextbox
			
			myEdit = FoundName
			MTB = myEdit.Tag
			Return myEdit.Text.Length >= MTB.MinChar
		End If
	End If
	Return False
End Sub

Private Sub IME_HandleAction As Boolean
Dim ET As View
Dim MTB As MyTextbox
Dim index As Int

	ET = Sender
	MTB = ET.Tag
	index = MyViews.IndexOf(ET)
	If index > -1 Then
		Select Case MTB.ActionBtn
			Case ActionBtn_Next
				If MyViews.Size > index + 1 Then
					ET = MyViews.Get(index + 1)
					ET.RequestFocus
				End If
			Case ActionBtn_Previous
				If index > 0 Then
					ET = MyViews.Get(index - 1)
					ET.RequestFocus
				End If
			Case Else ' Done, Go, Search, Send
				If MTB.ActionSub.Length > 0 Then CallSubDelayed(MTB.ActionSubModule, MTB.ActionSub)
		End Select
	End If
End Sub

Private Sub Edit_FocusChanged (HasFocus As Boolean)
	If HasFocus = True Then
		IME.ShowKeyboard(Sender)
	Else
		If MinMaxWarn = True Then
			Dim ET As EditText
			Dim MTB As MyTextbox
			Dim index As Int
			
			ET = Sender
			index = MyViews.IndexOf(ET)
			If index > -1 Then
				MTB = ET.Tag
				If ET.Text.Length < MTB.MinChar Then ToastMessageShow(MTB.Name & " Needs to be at least " & MTB.MinChar & " Characters Long", False)
			End If
		End If
	End If
End Sub

Private Sub Edit_TextChanged (Old As String, New As String)
Dim cursorPOS As Int
Dim ET As EditText
Dim MTB As MyTextbox
Dim index As Int

	If New.CompareTo(Old) <> 0 Then
		ET = Sender
		index = MyViews.IndexOf(ET)
		If index > -1 Then
			MTB = ET.Tag
			cursorPOS = ET.SelectionStart
			If MTB.MaxChar > 0 Then
				If New.Length > MTB.MaxChar AND MinMaxWarn = True Then ToastMessageShow("Too many Characters input", False)
				ET.Text = New.SubString2(0, Min(MTB.MaxChar, New.Length))
			End If
			ET.SelectionStart = Min(cursorPOS, ET.Text.Length)
		End If
	End If
End Sub